library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.config.all;

entity freq_sel is
    port (
        clk     : in  std_logic;
        reset   : in  std_logic;
        btn     : in  std_logic;                    -- button for selecting frequency of signal
        sw      : in  std_logic;                    -- switch for enabling the selection of frequency of signal
        count   : out integer range 0 to MAX_COUNT -- the number of division of cristal clock to generate the selected signal
    );
end freq_sel;

architecture Behavioral of freq_sel is
    type state_type is (selection, ready); 
    signal state_reg, state_next: state_type; 
    signal s_reg, s_next: integer range 0 to 3;
begin

-- FSMD state & data register
process(clk, reset)
begin
    if rising_edge(clk) then
        if reset = '1' then
            state_reg <= ready; 
            s_reg     <= 0; 
        else
            state_reg <= state_next;
            s_reg     <= s_next; 
        end if;
    end if;
end process; 

-- Next state logic & data path function units / routing
process(state_reg, s_reg, btn, sw)
begin
    state_next <= state_reg; 
    s_next <= s_reg; 

    case state_reg is 
        when ready =>
            if sw = '1' then
--                s_next <= 0; 
                state_next <= selection; 
            end if; 
            
        when selection =>
            if sw = '1' then 
                if (btn = '1') then
                    if (s_reg = 3) then
                        s_next <= 0; 
                    else
                        s_next <= s_reg + 1;
                    end if; 
                end if; 
            else
                state_next <= ready; 
            end if; 
    end case; 
    
end process; 

-- Output logic: start & count

with s_reg select
    count   <= 610  when 0,    -- 20Hz
               407  when 1,    -- 30Hz
               305  when 2,    -- 40Hz
               244  when 3,    -- 50Hz
               610  when others;
end Behavioral;
